// PRE-RELASE VERSION SUBJECT TO CHANGE
// Please do not distribute.
//
// Template Numerical Toolkit (TNT) for Linear Algebra 
// QR example
//
// R. Pozo
// Applied and Computational Mathematics Division
// National Institute of Standards and Technology

// Solve a linear system using QR factorization.
//
// Usage: a.out < matrix.dat
//
// where matrix.dat is an ASCII file consisting of the
// matrix size (M,N) followed by its values.  For example,
//
//  3  2
//  8.1  1.2  4.3
//  1.3  4.3  2.9
//
// 

#include <iostream.h>

#include "tnt.h"
#include "algomat.h"    
#include "qr.h"

main()
{
    Matrix_double A;

    cin >> A;

    Subscript N = A.dim(1);
    assert(N == A.dim(2));

    Vector_double b(N, 1.0);    // b= [1,1,1,...]
    Vector_double C(N), D(N);

    cerr << "A: " << A << endl;

    Matrix_double T(A);
    if (QR_factor(T, C, D) !=0)
    {
        cerr << "QR failed."  << endl;
        exit(1);
    }

    Vector_double x(b);
    if (QR_solve(T, C, D, x) == 1)
    {
        cerr << "QR_Solve did not work." << endl;
        exit(1);
    }

    cerr << "Solution x for Ax=b, where b=[1,1,...] " <<endl;
    cerr << " x: " << x << endl;

    cerr << "A*x should be the vector [1,1,...]; it is actually: "  <<endl;
    cerr     << "residual [A*x - b]: " << matmult(A, x)  - b << endl;

}
